//----------------------------------------------------------------------------
// 
//	file: matrix.h
// 
//	function: matrix template class 
//		Only works for scalars or structs that do not
//		need constructors.
//
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sat Oct 31 00:54:48 CSfloat 1998
// 
//	$Revision: 1.1 $ $Date: 1999/02/09 07:42:25 $
//	$Log: matrix.h,v $
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __MATRIX_H
#define __MATRIX_H

#include <stdlib.h>
#include <iostream.h>
#include "ppmio.h"

class matrix {

  int cols, rows;

public:

  float** mat;

  matrix(int c, int r) { mat = alloc_local_matrix(c,r); };
  ~matrix() { delete_matrix(); };

  void reset_matrix() {

  	// this may not work for non-scaler types
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      mat[i][j] = (float)0;
	   }
        }

  }

  void put_rgb_ppm(char *filename) {
	// create ppm object
  	ppmio *ppm = new ppmio(mat, cols, rows);

	// write out
	ppm->ppmwrite(filename,0.0,0.0,PPM_RGB);

	delete ppm;
  }	

  void put_hsv_ppm(char *filename) {
	// create ppm object
  	ppmio *ppm = new ppmio(mat, cols, rows);

	// write out
	ppm->ppmwrite(filename,0.0,0.0,PPM_HSV);

	delete ppm;
  }	

  void print_matrix() {

  	// the class must have "<<" defined
	for (int j=0; j<rows; ++j) {
	   for (int i=0; i<cols; ++i) {
	      cout << mat[i][j] << " ";
	   }
	   cout << endl;
        }

  }

  void print_cartesian_matrix() {
 
	// n
	// |
 	// |	  print in this format
	// 0 -- m  
  	// the class must have "<<" defined
	for (int j=rows-1; j>=0; --j) {
	   for (int i=0; i<cols; ++i) {
	      cout << mat[i][j] << " ";
	   }
	   cout << endl;
        }

  }

  static void print_matrix(float **mat, int cols, int rows) {

  	// the class must have "<<" defined
	for (int j=0; j<rows; ++j) {
	   for (int i=0; i<cols; ++i) {
	      cout << mat[i][j] << " ";
	   }
	   cout << endl;
        }

  }

  static void scaleup(float **mat, int cols, int rows, float scaleup) {

	for (int j=0; j<rows; ++j) {
	   for (int i=0; i<cols; ++i) {
	      mat[i][j] *= scaleup;
	   }
        }

  }

  void scaleup(float scale) {

	for (int j=0; j<rows; ++j) {
	   for (int i=0; i<cols; ++i) {
	      mat[i][j] *= scale;
	   }
        }

  }

  void random_matrix() {

  	// this may not work for non-scaler types
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      mat[i][j] = (float)drand48();
	   }
        }

  }

  void boolean_random_matrix() {

  	// this may not work for non-scaler types
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      mat[i][j] = (float)drand48()>0.5?1.0:0.0;
	   }
        }

  }

  void gray_random_matrix(float sigma) {

	int cx  = cols/2;
	int cy  = rows/2;

  	// this may not work for non-scaler types
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      float r = (float)drand48();
	      float g = exp(-((i-cx)*(i-cx)+(j-cy)*(j-cy))/sigma/sigma);
	      if (r>0.6*g) 
		mat[i][j] = 0.0;
	      else if (r>0.3*g)
		mat[i][j] = 0.5;
	      else 
		mat[i][j] = 1.0;
	   }
        }

  }
  void gaussian_random_matrix(float sigma) {

	int cx  = cols/2;
	int cy  = rows/2;

  	// this may not work for non-scaler types
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      mat[i][j] = drand48()<exp(-((i-cx)*(i-cx)+(j-cy)*(j-cy))/sigma/sigma) ? 1.0 : 0.0;
	   }
        }

  }
  void export_matrix(float** destination) {

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      destination[i][j] = mat[i][j];
	   }
	}
  } 	

  void import_matrix(float** source) {

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      mat[i][j] = source[i][j];
	   }
	}
  } 	

  // calculate dot product like a vector :
  //   return value = A dot B = Sum(c_ij), where c_ij = a_ij * b_ij;
  float vector_dot_product(float** B) {

	float c=0;

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      c += mat[i][j]*B[i][j];
	   }
	}

	return c;
  } 	

  // calculate dot product like a vector :
  //   return value = A dot B = Sum(c_ij), where c_ij = a_ij * b_ij;
  float vector_dot_product(float **A, float** B) {

	float c=0;

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      c += A[i][j]*B[i][j];
	   }
	}

	return c;
  } 	

  int get_cols() { return cols; }
  int get_rows() { return rows; }
  float get(int col, int row) { return mat[col][row]; }
  void put(int col, int row, float val) { mat[col][row]=val; }
  	
private: 

  float** alloc_local_matrix(int c, int r)  {
	
	float** temp;

	cols = c;
	rows = r;

	temp = new float* [cols];
	for (int i=0; i<cols; ++i) {
		temp[i] = new float [rows];
	}
	return temp;
  }

  void delete_matrix()  {
	
	for (int i=0; i<cols; ++i) {
		delete [] mat[i];
	}
	delete mat;
  }
};
#endif
