#!/usr/bin/perl

# The Math::CDF package needs to be installed to use
# the following. www.cpan.com
#----

require "getopts.pl";
&Getopts('s');

$usage = <<EOF;
usage: confint.pl [-s] [<CI>] <inputfile>

	-s 	    : use standard deviation instead of confidence interval

	<CI>	    : optional. the confidence interval [0..1]
		    Two-tailed interval will be calculated.
		    Default is 0.05, for 95% confidence interval. 
		    
	<inputfile> : 
		where each column of the input file corresponds to a single
		data vector.

	output: confidence interval for each dataset.
EOF

use Math::CDF qw(:all);

#----------------------------------------
#	check args
#----------------------------------------
if ($#ARGV==0) { # input only, set confidence interval to 0.05

	$p = 0.05;
	$inpfile = $ARGV[0];

} elsif ($#ARGV==1) { # CI and input file

	$p = $ARGV[0];
	$inpfile = $ARGV[1];

} else {
	print $usage;
	exit;
}

#----------------------------------------
#	load input and calculate sample mean and variance
#----------------------------------------

open(FP,$inpfile);
@inp = <FP>;
close(FP);

#--------------------
# remove leading blanks
#--------------------
for ($i=0; $i<=$#inp; ++$i) {
	$inp[$i] =~ s/^[\s]+//g; 
}

#--------------------
# calculate columns
#--------------------
$cols = split(/[\s]+/,$inp[0]);

#--------------------
# calc sample mean
#--------------------

for ($i=0; $i<$cols; ++$i) {
	$sum[$i] = 0;
}

$N = $#inp + 1;
# Sum
for ($i=0; $i<$N; ++$i) {
	@vals = split(/[\s]+/,$inp[$i]);
	for ($k=0; $k<$cols; ++$k) {
		$sum[$k] += $vals[$k];
	}	
}

# Mean
for ($i=0; $i<$cols; ++$i) {
	$sum[$i] /= $N;
	# print "$sum[$i] ";
}

#--------------------
# calc sample variance
#--------------------

for ($i=0; $i<$cols; ++$i) {
	$var[$i] = 0;
}

# Var
for ($i=0; $i<$N; ++$i) {
	@vals = split(/[\s]+/,$inp[$i]);
	for ($k=0; $k<$cols; ++$k) {
		$var[$k] += ($vals[$k]-$sum[$k])*($vals[$k]-$sum[$k]);
	}	
}

for ($i=0; $i<$cols; ++$i) {
	$var[$i] /= $N-1;
	# print "$var[$i] ";
}

#----------------------------------------
#	calculate quantile for the given CI
#----------------------------------------

$t = -qt($p/2,$N-1);

for ($i=0; $i<$cols; ++$i) {
	if ($opt_s) {
		$low = $sum[$i] - sqrt($var[$i]);
		$high = $sum[$i] + sqrt($var[$i]);
	} else {
		$low = $sum[$i] - $t * sqrt( $var[$i] / $N );
		$high = $sum[$i] + $t * sqrt( $var[$i] / $N );
	}
	print "$sum[$i] $low $high ";
}
print "\n";


