## usage: [Emax,Vmax,Eimg] = convedgeplot(Img, kernelsize, freq)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		kernelsize: width (==height) of convolution kernel
##		freq: 	    gabor frequency (relative to the kernelsize) 
##
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of orientations 0, 10, 20, ... , 180. 
##
##	Usage example:
##		[Emax,Vmax,Eimg] = convedgeplot(imagemat,9,1);
##
##		both DoG and gabor filters are  9 x 9, and
##		Gabor frequency is 1.
##		final grid sampling size is 3 x 3.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [Emax,Vmax,Eimg] = convedgeplot(Img, kernelsize, freq)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(kernelsize,2) == 0 ) 
		kernelsize ++;
	endif

	DoG = dog(kernelsize);

        # convolve with DoG, and only use the real part
	Eimg = real(conv2(Img,DoG,"same"));	
  
       	Vmax=zeros(oldp,oldq);
        Emax=zeros(oldp,oldq);
       
	g = gabor(kernelsize,kernelsize,"sin",0,freq);

        Vmax=real(conv2(Eimg,g,"same"));
	Emax=Emax+1;
        
        #find Vmax and Emax iteratively
	for k=2:19
             g = gabor(kernelsize,kernelsize,"sin",(k-1)*10,freq);
             Vmax=max(Vmax,real(conv2(Eimg,g,"same")));
             Emax=max(Emax,k*(sign((real(conv2(Eimg,g,"same")))-Vmax)+1));
        endfor


endfunction
