//----------------------------------------------------------------------------
// 
//	file: ppmatrgb.cc
// 
//	function: create a ppm file from the three files - rcolor.dat gcolor.dat and bcolor.dat
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Thu Feb 07 06:14:52 CST 2002
// 
//	$Revision: $ $Date: $
//	$Log: $
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <stdio.h>
#include "../ppmio.h"
#include "../matrix.h"
#include "nnview/filename.h"
#include "nnview/real.h"

void getcolrows(FILE *fp,int *rowsr, int *colsr);

//----------------------------------------------------------------------------
//	usage() 
//	  - prog usage
//----------------------------------------------------------------------------
void usage(int argc) {

  if (argc != 2) { 
    cout  << " usage: ppmmatrgb <outputfile>" << endl;
     exit(0);
  }

}

//----------------------------------------------------------------------------
//	draw_bar() 
//	  - fill in given rectangle with a value
//----------------------------------------------------------------------------
void draw_bar(int col, int row, int width, int height,
	REAL value, REAL **dest) {
 

  int start = col*width;
  int start_row = row*height;

  for (int i=start; i<start+width; ++i) 
     for (int j=start_row; j<start_row+height; ++j)
		dest[i][j] = value;

}
  
//----------------------------------------------------------------------------
//	main() 
//	  - the main
//----------------------------------------------------------------------------
int main(int argc, char *argv[]) {

  //   usage(argc);
	
  FILE *rfp;
  FILE *gfp;
  FILE *bfp;
  FILE *hfp;
  FILE *sfp;
  FILE *vfp;
  FILE *outfile;
  char *fnrgb = "testrgb.ppm";
  char *fnhsv = "testhsv.ppm";
  char ch;

  int colsr,rowsr;
  int colsg, rowsg;
  int colsb, rowsb;
  int colsh,rowsh;
  int colss, rowss;
  int colsv, rowsv;
  int x,y;
  REAL *addr;
  float tempval;

  rfp=fopen("rcolor.dat","r");
  gfp=fopen("gcolor.dat","r");
  bfp=fopen("bcolor.dat","r");
  hfp=fopen("hcolor.dat","r");
  sfp=fopen("scolor.dat","r");
  vfp=fopen("vcolor.dat","r");
  
  getcolrows(rfp, &rowsr, &colsr);
  rowsg = rowsb = rowsh = rowss = rowsv = rowsr;
  colsg = colsb = colsh = colss = colsv = colsr;
//   getcolrows(gfp, &rowsg, &colsg);
//   getcolrows(bfp, &rowsb, &colsb);
//   
//   getcolrows(hfp, &rowsh, &colsh);
//   getcolrows(sfp, &rowss, &colss);
//   getcolrows(vfp, &rowsv, &colsv); 


  fclose(rfp);
  fclose(gfp);
  fclose(bfp);
  fclose(hfp);
  fclose(sfp);
  fclose(vfp);
  

  if (((colsr!=colsg) || (colsg!=colsb)||(colsb!=colsr))||
      ((rowsr!=rowsg) || (rowsg!=rowsb)||(rowsb!=rowsr)))
    {
      cout << "Row / Column information for the three R,G,B files do not match." << endl;
      cout << "Please check the files" << endl;
      exit(0);
    }
 
  if (((colsh!=colss) || (colss!=colsv)||(colsv!=colsh))||
      ((rowsh!=rowss) || (rowss!=rowsv)||(rowsv!=rowsh)))
    {
      cout << "Row / Column information for the three H,S,V files do not match." << endl;
      cout << "Please check the files" << endl;
      exit(0);
    }
  int ppm_width=colsr;
  int ppm_height=rowsr;
 
  int ppm_width_hsv=colsh;
  int ppm_height_hsv=rowsh;
 
  matrix<REAL> *ppm_buff_r= new matrix<REAL>(ppm_width,ppm_height);
  matrix<REAL> *ppm_buff_g= new matrix<REAL>(ppm_width,ppm_height);
  matrix<REAL> *ppm_buff_b= new matrix<REAL>(ppm_width,ppm_height);
  
  matrix<REAL> *ppm_buff_h= new matrix<REAL>(ppm_width_hsv,ppm_height_hsv);
  matrix<REAL> *ppm_buff_s= new matrix<REAL>(ppm_width_hsv,ppm_height_hsv);
  matrix<REAL> *ppm_buff_v= new matrix<REAL>(ppm_width_hsv,ppm_height_hsv);

  rfp=fopen("rcolor.dat","r");
  gfp=fopen("gcolor.dat","r");
  bfp=fopen("bcolor.dat","r");
  
  hfp=fopen("hcolor.dat","r");
  sfp=fopen("scolor.dat","r");
  vfp=fopen("vcolor.dat","r");

  // get data from red color dat file
  for (y=0; y<rowsr; ++y)
  {   for (x=0; x<colsr; ++x)
       {
       	
	fscanf(rfp,"%f",&tempval);
        ppm_buff_r->mat[x][y]=tempval;
        fscanf(rfp,"%c",&ch);
       }
   fscanf(rfp,"%c",&ch);
  }
 // get data from green color dat file
  for (y=0; y<rowsg; ++y)
    { 
    for (x=0; x<colsg; ++x)
       {
	fscanf(gfp,"%f",&tempval);
        ppm_buff_g->mat[x][y]=tempval;
         fscanf(gfp,"%c",&ch);
       }
     fscanf(gfp,"%c",&ch);
    }
 // get data from blue color dat file
  for (y=0; y<rowsb; ++y)
    {
     for (x=0; x<colsb; ++x)
        {
	fscanf(bfp,"%f",&tempval);
        ppm_buff_b->mat[x][y]=tempval;
         fscanf(bfp,"%c",&ch);
        }
       fscanf(bfp,"%c",&ch);
    } 

    // get data from hue color dat file
  for (y=0; y<rowsh; ++y)
  {   for (x=0; x<colsh; ++x)
       {
       	
	fscanf(hfp,"%f",&tempval);
        ppm_buff_h->mat[x][y]=tempval;
        fscanf(hfp,"%c",&ch);
       }
   fscanf(hfp,"%c",&ch);
  }
 // get data from saturation color dat file
  for (y=0; y<rowss; ++y)
    { 
    for (x=0; x<colss; ++x)
       {
	fscanf(sfp,"%f",&tempval);
        ppm_buff_s->mat[x][y]=tempval;
         fscanf(sfp,"%c",&ch);
       }
     fscanf(sfp,"%c",&ch);
    }
 // get data from value color dat file
  for (y=0; y<rowsv; ++y)
    {
     for (x=0; x<colsv; ++x)
        {
	fscanf(vfp,"%f",&tempval);
        ppm_buff_v->mat[x][y]=tempval;
         fscanf(vfp,"%c",&ch);
        }
       fscanf(vfp,"%c",&ch);
    } 
  
   ppmio *io = new ppmio(ppm_buff_r->mat,ppm_buff_g->mat,ppm_buff_b->mat,ppm_width,ppm_height,256);  
 
 
  // write matrix to ppm file
  io->ppmwrite(fnrgb,0.0,0.0,PPM_TRUE_RGB,PPM_NOFLIP);
  ch=' ';
  ppmio *io_hsv = new ppmio(ch,ppm_buff_h->mat,ppm_buff_s->mat,ppm_buff_v->mat,ppm_width_hsv,ppm_height_hsv,256);

  io_hsv->hsv_to_rgb();
 // write converted matrix to ppm file
  io_hsv->ppmwrite(fnhsv,0.0,0.0,PPM_TRUE_RGB,PPM_NOFLIP);
    
  delete io;
  delete io_hsv;
  delete ppm_buff_r;
  delete ppm_buff_g;
  delete ppm_buff_b;
  delete ppm_buff_h;
  delete ppm_buff_s;
  delete ppm_buff_v;
  
  fclose(rfp);
  fclose(gfp);
  fclose(bfp);
  fclose(hfp);
  fclose(sfp);
  fclose(vfp);
  
  return(0);

}

void getcolrows(FILE *fp,int *rowsr, int *colsr)
{
 
  *rowsr=0;
  int val=0;
  int prevcols=0;

  float tt;
  char ch;
  
  
  while (!feof(fp))
  {
    ch=' ';
    prevcols=*colsr;
    *colsr=0;
   while (ch!='\n')
  {
    fscanf(fp,"%f",&tt);
    fscanf(fp,"%c",&ch);
    *colsr+=1;
    fscanf(fp,"%c",&ch);
    if (feof(fp)!=0)
      {
	//*rowsr+=1;
        *colsr=prevcols;
  	cout << "rows = " << *rowsr << ", cols = " << *colsr << endl;
	return;
      }
     
    if (ch!='\n')
      {
      ungetc(ch,fp);
      ch=' ';
      }
  
  }
  *rowsr+=1;
  }

  cout << "rows = " << *rowsr << ", cols = " << *colsr << endl;
  return;

}
