#!/usr/bin/perl

# process (theta,d, hue) statistics dump from the command
# dump_wgt_or_stat in slissom.
#	Blank out every other track. Just simply multiply rho by 2.

$distance_binsize = 5;

if ($#ARGV != 3) {

	print "usage: prowgtstat.pl -hue|-huepolar|-prob|-max|-sum|-maxpolar|-probpolar|-freq_thetadelta_probpolar|-sumpolar|-hist <size> <thresh> <or_stat_dump_filename>\n";
	print "		-max is the default \n";
	print <<EOF;
	-hue : print the most probable hue -- 2d matrix
	-huepolar : print the most probable hue -- polar coordinate
	-prob  : print the probability -- 2d matrix
	-max   : prob of most probable hue -- 2d matrix
	-sum	: sum?
	-maxpolar : prob of most probable hue -- polar coordinate
	-probpolar : prob of most probable hue -- polar coordinate
        -freq_thetadelta_probpolar : prob of freq/thetadelta -- polar coordinate
	-sumpolar : ??
	<size> : only used for 2d matrix -- resulting size
EOF

	exit;

}

$hue_output_flag = ($ARGV[0] eq "-hue")?1:0;
$hue_polar_output_flag = ($ARGV[0] eq "-huepolar")?1:0;
$max_output_flag = ($ARGV[0] eq "-max")?1:0;
$max_polar_output_flag = ($ARGV[0] eq "-maxpolar")?1:0;
$sum_output_flag = ($ARGV[0] eq "-sum")?1:0;
$prob_polar_output_flag = ($ARGV[0] eq "-probpolar")?1:0;
$freq_thetadelta_prob_polar_output_flag = ($ARGV[0] eq "-freq_thetadelta_probpolar")?1:0;
$sum_polar_output_flag = ($ARGV[0] eq "-sumpolar")?1:0;
$hist_output_flag = ($ARGV[0] eq "-hist")?1:0;

# init matrix
$N = $ARGV[1];
for ($i=0; $i<$N; ++$i) {
	for ($j=0; $j<$N; ++$j) {
		$matrix[$i][$j] = 0;
	}
}

$thresh = $ARGV[2];

# load file and proces
open(FP,"$ARGV[3]");

$sum = 0.0;
$alternate_zero_degree = 0;
$alternate_count = 0;
$max_d = 0;
while(<FP>) {
	chop;
	# Get stat
	($theta,$d,$hue) = split(/\ /,$_);
	$theta = int($theta);
	$hue = int($hue);
	$d= int($d);

	$d = int($d/$distance_binsize);

	if ($d>$max_d) { $max_d = $d; }

	# Fix side effect of only having 0 to 170 degree
	if (0) {
		if ($alternate_count%2==0) {
			$hue = 180;
		} 
		$alternate_count ++;
	}


	# update theta,d,hue count
	$theta_d{"$theta $d $hue"} ++;
	$theta_d_sum{"$theta $d"} ++;
	$sum ++;
	$sum_prob{"$theta $d"} ++;
	
	# update max prob hue (hue that has the maximum count)
	if (exists($max_prob_hue{"$theta $d"})) {
		$max_hue = $max_prob_hue{"$theta $d"};
	} else {
		$max_hue = 0;
	}
	if ($theta_d{"$theta $d $hue"}>=$theta_d{"$theta $d $max_hue"}) {
		$max_prob_hue{"$theta $d"} = $hue;
		$max_prob{"$theta $d"} = $theta_d{"$theta $d $hue"};
	}
	# $sum_prob{"$theta $d"} += $theta_d{"$theta $d $hue"};


	# Dirty hack : rotate 180 and add that too.

	$theta = ($theta+180)%360;

	# update theta,d,hue count
	$theta_d{"$theta $d $hue"} ++;
	$theta_d_sum{"$theta $d"} ++;
	$sum ++;
	$sum_prob{"$theta $d"} ++;
	
	# update max prob hue (hue that has the maximum count)
	if (exists($max_prob_hue{"$theta $d"})) {
		$max_hue = $max_prob_hue{"$theta $d"};
	} else {
		$max_hue = 0;
	}
	if ($theta_d{"$theta $d $hue"}>=$theta_d{"$theta $d $max_hue"}) {
		$max_prob_hue{"$theta $d"} = $hue;
		$max_prob{"$theta $d"} = $theta_d{"$theta $d $hue"};
	}
	# $sum_prob{"$theta $d"} += $theta_d{"$theta $d $hue"};


	$theta = ($theta+180)%360;
}
close(FP);

print STDERR "ALT_COUNT (0 vs 180) = $alternate_count\n";

$thetadsum = 0.0;
foreach $key (keys(%sum_prob)) {
	$thetadsum += $sum_prob{$key};
}
print STDERR " SUM = $sum \n";
print STDERR " Sigma_{theta} Sigma_{d} Sigma_{hue} = $thetadsum \n";
  

if ($max_output_flag) {
	foreach $key (keys(%max_prob_hue)) {
		print "$key $max_prob_hue{$key}\n";
	}
	exit;
}

if ($hist_output_flag) {
    $d = 1;
    $theta = 350;
    for ($hue1 = 0; $hue1 <= 500; $hue1++) {
	if (exists($theta_d{"$theta $d $hue1"})) {
	$val1 = $theta_d{"$theta $d $hue1"};
        print "$hue1 $val1\n";
    }
    }
    exit;
}


# print result in a NxN matrix
$half_N = int($N/2);

$prob_sum = 0.0;
$min_prob = 999;
foreach $key (keys(%max_prob_hue)) {
	($theta,$d) = split(/\ /,$key);

  	if ($max_polar_output_flag) {
          		#$prob = $max_prob{"$theta $d"}/$sum_prob{"$theta $d"};
	                $prob = $max_prob{"$theta $d"}/$sum;
                        print "$theta $d $prob\n";
	} elsif ($prob_polar_output_flag) {
			$prob = $sum_prob{"$theta $d"}/($sum*1.0);
			if ($prob<$min_prob) { $min_prob = $prob; }
			$prob_sum += $prob;
			print "$theta $d $prob\n";
        } elsif ($freq_thetadelta_prob_polar_output_flag) {
			$prob = $max_prob{"$theta $d"}/$sum_prob{"$theta $d"};
			print "$theta $d $prob\n";
	} elsif ($sum_polar_output_flag) {
			$prob = $sum_prob{"$theta $d"};
			$prob_sum += $prob;
			print "$theta $d $prob\n";
	} elsif ($hue_polar_output_flag) {
			$hue = $max_prob_hue{$key};
			#$or_val = 1.0-$theta/180.0;
			$or_val = $hue;
			print "$theta $d $or_val\n";
	} else {
		# For cartesian matrix output
		$t = 3.141592*$theta/180.0;
		$x = int($d*cos($t)+0.5)+$half_N;
		$y = int($d*sin($t)+0.5)+$half_N;
		$hue = $max_prob_hue{$key};
		if ($max_prob{"$theta $d"}>$thresh) {
			if ($hue_output_flag || $hue_polar_output_flag) {
				$matrix[$x][$y] = $hue;
				
			} elsif ($sum_output_flag) {
				$matrix[$x][$y] = $sum_prob{"$theta $d"};
			} else {
				$matrix[$x][$y] = $max_prob{"$theta $d"};
			}
		}
	}
} 

# fill in ------ disabled
if (0) {
if ($prob_polar_output_flag) {
	for ($i=0; $i<360; $i++) {
		for ($d=0; $d<=$max_d; ++$d) {
			if(!exists($max_prob_hue{"$theta $d"})) {
				print "$i $d $min_prob\n";
			}
		}
	}
}	
}
		
		

print STDERR "RECOUNTED_SUM = $prob_sum \n";

if ($max_polar_output_flag || $prob_polar_output_flag || $sum_polar_output_flag || $hue_polar_output_flag)  {
	exit;
}

for ($j=0; $j<$N; ++$j) {
	for ($i=0; $i<$N; ++$i) {
		print $matrix[$i][$j]." ";
	}
	print "\n";
}

