#!/usr/bin/perl

use POSIX;

# process (phi,theta,d) statistics dump from the command
# dump_wgt_or_stat in slissom.
#	Blank out every other track. Just simply multiply rho by 2.

$distance_binsize = 5;

if ($#ARGV != 3) {

	print "usage: prowgtstat.pl -phi|-phipolar|-prob|-max|-sum|-maxpolar|-probpolar|-phi_thetadelta_probpolar|-sumpolar|-thetadeltapolar|-specialpolar|-hist|-rawstatpolar <size> <thresh> <or_stat_dump_filename>\n";
	print "		-max is the default \n";
	print <<EOF;
	-phi : print the most probable phi -- 2d matrix
	-phipolar : print the most probable phi -- polar coordinate
	-prob  : print the probability -- 2d matrix
	-max   : prob of most probable phi -- 2d matrix
	-sum	: sum?
	-maxpolar : prob of most probable phi -- polar coordinate
	-probpolar : prob of most probable phi -- polar coordinate
        -phi_thetadelta_probpolar : prob of phi/theta,delta -- polar coordinate
	-sumpolar : ??
        -thetadeltapolar : prob of every theta,delta -- polar coordinate
	-rawstatpolar : print joint prob for all phi,theta,delta -- polar coordinate
	<size> : only used for 2d matrix -- resulting size
EOF

	exit;

}

$phi_output_flag = ($ARGV[0] eq "-phi")?1:0;
$phi_polar_output_flag = ($ARGV[0] eq "-phipolar")?1:0;
$max_output_flag = ($ARGV[0] eq "-max")?1:0;
$max_polar_output_flag = ($ARGV[0] eq "-maxpolar")?1:0;
$sum_output_flag = ($ARGV[0] eq "-sum")?1:0;
$prob_polar_output_flag = ($ARGV[0] eq "-probpolar")?1:0;
$phi_thetadelta_prob_polar_output_flag = ($ARGV[0] eq "-phi_thetadelta_probpolar")?1:0;
$sum_polar_output_flag = ($ARGV[0] eq "-sumpolar")?1:0;
$thetadelta_polar_output_flag = ($ARGV[0] eq "-thetadeltapolar")?1:0;
$special_polar_output_flag = ($ARGV[0] eq "-specialpolar")?1:0;
$raw_stat_polar_output_flag = ($ARGV[0] eq "-rawstatpolar")?1:0;
$hist_output_flag = ($ARGV[0] eq "-hist")?1:0;

# init matrix
$N = $ARGV[1];
for ($i=0; $i<$N; ++$i) {
	for ($j=0; $j<$N; ++$j) {
		$matrix[$i][$j] = 0;
	}
}

$thresh = $ARGV[2];

# load file and proces
open(FP,"$ARGV[3]");

$sum = 0.0;
$alternate_zero_degree = 0;
$alternate_count = 0;
$max_d = 0;
while(<FP>) {
	chop;
	# Get stat
	($phi,$d,$theta) = split(/\ /,$_);
	$phi = int($phi/10)*10;
	$theta = int($theta);
	$d= int($d);

	$d = ceil($d/$distance_binsize);

	if ($d>$max_d) { $max_d = $d; }

	# Fix side effect of only having 0 to 170 degree
	if (0) {
		if ($alternate_count%2==0) {
			$theta = 180;
		} 
		$alternate_count ++;
	}


	# update phi,d,theta count
	$theta_d{"$phi $d $theta"} ++;
	$theta_d_sum{"$d $theta"} ++;
        $d_sum{"$d"} ++;
	$sum ++;
	$sum_prob{"$d $theta"} ++;
	
	# update max prob phi (phi that has the maximum count)
	if (exists($max_prob_phi{"$d $theta"})) {
		$max_phi = $max_prob_phi{"$d $theta"};
	} else {
		$max_phi = 0;
	}
	if ($theta_d{"$phi $d $theta"}>=$theta_d{"$max_phi $d $theta"}) {
		$max_prob_phi{"$d $theta"} = $phi;
		$max_prob{"$d $theta"} = $theta_d{"$phi $d $theta"};
	}
       

	# $sum_prob{"$phi $d"} += $phi_d{"$phi $d $theta"};

	# Dirty hack : rotate 180 and add that too.

	$theta = ($theta+180)%360;

	# update phi,d,theta count
	$theta_d{"$phi $d $theta"} ++;
	$theta_d_sum{"$d $theta"} ++;
	$d_sum{"$d"} ++;
        $sum ++;
	$sum_prob{"$d $theta"} ++;
	
	# update max prob phi (phi that has the maximum count)
	if (exists($max_prob_phi{"$d $theta"})) {
		$max_phi = $max_prob_phi{"$d $theta"};
	} else {
		$max_phi = 0;
	}
	if ($theta_d{"$phi $d $theta"}>=$theta_d{"$max_phi $d $theta"}) {
		$max_prob_phi{"$d $theta"} = $phi;
		$max_prob{"$d $theta"} = $theta_d{"$phi $d $theta"};
	}
	# $sum_prob{"$phi $d"} += $phi_d{"$phi $d $theta"};
        

	$theta = ($theta+180)%360;
}
close(FP);

print STDERR "ALT_COUNT (0 vs 180) = $alternate_count\n";


$thetadsum = 0.0;
foreach $key (keys(%sum_prob)) {
	$thetadsum += $sum_prob{$key};
}
print STDERR " SUM = $sum \n";
print STDERR " Sigma_{phi} Sigma_{d} Sigma_{theta} = $thetadsum \n";
  

if ($max_output_flag) {
	foreach $key (keys(%max_prob_phi)) {
	        #$max_prob_phi{$key}=int($max_prob_phi{$key});
		print "$max_prob_phi{$key} $key\n";
	}
	exit;
}

if ($thetadelta_polar_output_flag) {
	foreach $key (keys(%sum_prob)) {
                ($d,$theta) = split(/\ /,$key);
                $prob = $sum_prob{"$d $theta"}/$sum;
		print "$theta $d $prob\n";
	}
	exit;
}

if ($special_polar_output_flag) {
    $d = 2;
    $theta = 180;
    $phi = 172;

    $prob_phi_dtheta = $theta_d{"$phi $d $theta"}/$sum_prob{"$d $theta"};
    print "P(phi/d theta) = $prob_phi_dtheta\n";
   
    $prob1 = $sum_prob{"$d $theta"}/$sum;
  
    print "P(d theta) = $prob1\n";
    exit;

}

if ($hist_output_flag) {
    $d = 13;
    $theta = 0;
    for ($phi1 = 0; $phi1 <= 180; $phi1++) {
	if (exists($theta_d{"$phi1 $d $theta"})) {
	$val1 = $theta_d{"$phi1 $d $theta"};
        print "$phi1 $val1\n";
    }
    }
    exit;
}

if ($raw_stat_polar_output_flag) {
  foreach $key (keys(%theta_d)) {
	print $key." ".($theta_d{"$key"}/$sum)."\n";
  }
  exit;
}

   
# print result in a NxN matrix
$half_N = int($N/2);

$prob_sum = 0.0;
$min_prob = 999;
foreach $key (keys(%max_prob_phi)) {
	($d,$theta) = split(/\ /,$key);

  	if ($max_polar_output_flag) {
	                #$max_prob{"$d $theta"}=int($max_prob{"$d $theta"});
                        #$max_prob{"$d $theta"}=$max_prob{"$d $theta"};
			#print $max_prob{"$d $theta"}." $d $theta"."\n";
                        #$prob1 = $d_sum{"$d"};
                        #print STDERR "d= $d , Count of d = $prob1\n";
                        $prob = $max_prob{"$d $theta"}/$sum;
                        print "$theta $d $prob\n";
	} elsif ($prob_polar_output_flag) {
			$prob = $sum_prob{"$d $theta"}/($sum*1.0);
	                #$prob = $max_prob{"$d $theta"}/$sum_prob{"$d $theta"};
	                #$prob = $max_prob{"$d $theta"};
                        if ($prob<$min_prob) { $min_prob = $prob; }
			$prob_sum += $prob;
			print "$theta $d $prob\n";
        } elsif ($phi_thetadelta_prob_polar_output_flag) {
			$prob = $max_prob{"$d $theta"}/$sum_prob{"$d $theta"};
	              	print "$theta $d $prob\n";
	} elsif ($sum_polar_output_flag) {
			$prob = $sum_prob{"$d $theta"};
			$prob_sum += $prob;
			print "$theta $d $prob\n";
                        
	} elsif ($phi_polar_output_flag) {
                        
	                #$max_prob_phi{$key}=int($max_prob_phi{$key});
			$phi = $max_prob_phi{$key};
			#$or_val = 1.0-$phi/180.0;
                        #$or_val = 1.0-$phi/360.0;
			$or_val = $phi;
                        print "$or_val $d $theta\n";
	                #print "$phi $d $theta\n";
        } else {
		# For cartesian matrix output
		$t = 3.141592*$theta/180.0;
		$x = int($d*cos($t)+0.5)+$half_N;
		$y = int($d*sin($t)+0.5)+$half_N;
                $max_prob_phi{$key}=int($max_prob_phi{$key});
		$phi = $max_prob_phi{$key};
		if ($max_prob{"$d $theta"}>$thresh) {
			if ($phi_output_flag || $phi_polar_output_flag) {
				$matrix[$x][$y] = 1.0-$phi/180.0;
				
			} elsif ($sum_output_flag) {
				$matrix[$x][$y] = $sum_prob{"$d $theta"};
			} else {
				$matrix[$x][$y] = $max_prob{"$d $theta"};
			}
		}
	}
} 

# fill in ------ disabled
if (0) {
if ($prob_polar_output_flag) {
	for ($i=0; $i<360; $i++) {
		for ($d=0; $d<=$max_d; ++$d) {
			if(!exists($max_prob_phi{"$d $theta"})) {
				print "$i $d $min_prob\n";
			}
		}
	}
}	
}
		
		

print STDERR "RECOUNTED_SUM = $prob_sum \n";

if ($max_polar_output_flag || $prob_polar_output_flag || $sum_polar_output_flag || $phi_polar_output_flag ) {
	exit;
}

for ($j=0; $j<$N; ++$j) {
	for ($i=0; $i<$N; ++$i) {
		print $matrix[$i][$j]." ";
	}
	print "\n";
}

