## Copyright (C) 2000 uschoe
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: [t g] = probcoords(M)
##
## Returns a 2xN matrix with each column containing (x,y) coordinates
## calculated from the 2D matrix M. Each column (x,y) is added if M(y,x)
## is <= rand;
## t : contains the 2xN matrix
## g : 2D Matrix with the same size as M with 0 or 1 
##

## Author: Yoonsuck Choe <yschoe@cs.utexas.edu>

function [t g] = probcoords (M)

	# scale matrix M between 0 and 1
	minval = min(vec(M));
	maxval = max(vec(M));
	range = maxval - minval;
	m = (M.-min(vec(M)))./range;

	# compare with rand
	r = rand(rows(M),columns(M));

	g = ceil(thresh(m-r,0));

	mrows = rows(M);
	mcols = columns(M);

	t = [ 0 0 ];
	for i = 1:mrows
		for j = 1:mcols
			if (g(i,j)==1) 
				t = [ t ; [j i] ];
			endif
		endfor
	endfor
		
endfunction
