#!/usr/bin/wish

global E nV G screen_w screen_h cols rows zoom groupcount maxgroup pressed
global colorlist pressed2 saved

set pressed 0
set pressed2 0
set zoom 2
set saved 0

set colorlist [list yellow red blue yellow green magenta cyan ]

#----------------------------------------------------------------------------
# 	flash current group
#----------------------------------------------------------------------------
proc showgroup { groupnum } {

	global E nV screen_w screen_h cols rows zoom groupcount maxgroup

}

#----------------------------------------------------------------------------
# 	print G
#----------------------------------------------------------------------------
proc printG { } {

	global E G nV screen_w screen_h cols rows zoom groupcount maxgroup saved

	puts "Saving to file tagG"

	set fp [open tagG "w"];
	for {set j 0} {$j<$rows} {incr j} {
		for {set i 0} {$i<$cols} {incr i} {
			puts -nonewline $fp $G($j-$i)
			puts -nonewline $fp " "
		}
		puts $fp " "
	}
	close $fp	
	
}


#----------------------------------------------------------------------------
# 	group edges
#----------------------------------------------------------------------------
proc group { x y } {

	global E nV G screen_w screen_h cols rows zoom 
	global saved groupcount maxgroup colorlist

 	set granx [expr $screen_w / $cols];
 	set grany [expr $screen_h / $rows];
	

	set size [expr $granx/2];

	# set cx [expr $x * $granx]
	# set cy [expr $y * $grany]
	set cx [expr int($x/$granx)*$granx]
	set cy [expr int($y/$grany)*$grany]

	set idx [expr int($x/$granx)]
	set idy [expr int($y/$grany)]
	
	# set color to the current group
	set c [lindex $colorlist [expr $groupcount % 7]];

	if { $nV($idy-$idx)>0.0 && $G($idy-$idx)==0 } {
	  .misc.net create oval \
		[expr $cx-$size] \
		[expr $cy-$size] \
		[expr $cx+$size] \
		[expr $cy+$size] \
		-outline $c -tags [list "$idx-$idy"]
	  set G($idy-$idx) $groupcount	
	  set saved 0
	} 

	update
}

#----------------------------------------------------------------------------
# 	ungroup edges
#----------------------------------------------------------------------------
proc ungroup { x y } {

	global E G nV screen_w screen_h cols 
	global saved rows zoom groupcount maxgroup

 	set granx [expr $screen_w / $cols];
 	set grany [expr $screen_h / $rows];
	
	# set cx [expr $x * $granx]
	# set cy [expr $y * $grany]
	set cx [expr int($x/$granx)*$granx]
	set cy [expr int($y/$grany)*$grany]

	set idx [expr int($x/$granx)]
	set idy [expr int($y/$grany)]
	
	.misc.net addtag currentEdge withtag "$idx-$idy"
	.misc.net delete currentEdge

	set G($idy-$idx) 0
	set saved 0
	update
}

#----------------------------------------------------------------------------
# 	plotnet: plot c.o.g.s net 
#----------------------------------------------------------------------------
proc plotnet { } {

	global E nV screen_w screen_h cols rows saved
	
 	set granx [expr $screen_w / $cols];
 	set grany [expr $screen_h / $rows];
	set length $granx

	for {set i 0} {$i<$rows} {incr i} {
	    for {set j 0} {$j<$cols} {incr j} {

		if {$nV($i-$j)>0.0} {

			set cx [expr $j*$granx]
			set cy [expr $i*$grany]

			set theta [expr $E($i-$j)-1]

			set rad_theta [expr $theta*3.141592/18.0]
			set offsetx [expr $length * cos($rad_theta)]
			set offsety [expr $length * sin($rad_theta)]

			.misc.net create line   \
				[expr $cx-$offsetx/2.0] \
				[expr $cy+$offsety/2.0] \
				[expr $cx+$offsetx/2.0] \
				[expr $cy-$offsety/2.0] \
				-fill \#000000 -width 1 
		}
	    }
	}
}


#----------------------------------------------------------------------------
#
# 	MAIN:
#
#----------------------------------------------------------------------------
if {[llength $argv] <= 1} {
  puts "usage: tagedges.tcl <Efile> <nVfile> <ppmfile> \[<zoom:default 8>\]"
  exit
}

if {[llength $argv] == 4} {
	set zoom [lindex $argv 3]
	# puts "zoom: $zoom"
}

#----------------------------------------
# 1. Load E data
#----------------------------------------
set fp [open [lindex $argv 0]]

set max_e 0
set idx 0

#--------------------
# skip octave comments
#--------------------
set line [gets $fp]
set line [gets $fp]
set line [gets $fp]

#--------------------
# get rows
#--------------------
set line [gets $fp]
set rows [lindex [split $line " "] 2]
# puts "rows: $rows"

#--------------------
# get columns
#--------------------
set line [gets $fp]
set cols [lindex [split $line " "] 2]
# puts "cols: $cols"

#--------------------
# get data
#--------------------
set j 0

while {![eof $fp]} {
  
  set line [gets $fp]

  if {[eof $fp]} {
		break
  }

  set column_data [split $line " "]

  for {set i 0} {$i<$cols} {incr i} {
	set E($j-$i) [lindex $column_data [expr $i+1]]
	set G($j-$i) 0
  }

  incr j
}
close $fp

#----------------------------------------
# 2. Load nV data
#----------------------------------------
set fp [open [lindex $argv 1]]

set max_e 0
set idx 0

#--------------------
# skip octave comments
#--------------------
set line [gets $fp]
set line [gets $fp]
set line [gets $fp]

#--------------------
# get rows
#--------------------
set line [gets $fp]
set rows [lindex [split $line " "] 2]
# puts "rows: $rows"

#--------------------
# get columns
#--------------------
set line [gets $fp]
set cols [lindex [split $line " "] 2]
# puts "cols: $cols"

#--------------------
# get data
#--------------------
set j 0

while {![eof $fp]} {
  
  set line [gets $fp]

  if {[eof $fp]} {
		break
  }

  set column_data [split $line " "]

  for {set i 0} {$i<$cols} {incr i} {
	set nV($j-$i) [lindex $column_data [expr $i+1]]
  }

  incr j
}
close $fp

#----------------------------------------
# 2. Create polar line display window
#----------------------------------------
label .inst -relief ridge -text "Mouse Buttons: Left (group); Middle (ungroup); Right (start new group)"; pack .inst -padx 10 -pady 10
frame .misc -relief ridge; pack .misc -padx 10 -pady 10

set ppmfilename [lindex $argv 2]
image create photo BGPPMsmall -format ppm -file $ppmfilename
set img_w [image width BGPPMsmall]
set img_h [image height BGPPMsmall]

set screen_w [expr $img_w * $zoom]
set screen_h [expr $img_h * $zoom]

canvas .misc.net -relief sunken -borderwidth 1 -background white \
	-width $screen_w -height $screen_h;
pack .misc.net -side left -padx 10 -pady 5 ; 

set imgzoomx [expr $screen_w/$img_w];
set imgzoomy [expr $screen_h/$img_h];
# puts "$imgzoomx $imgzoomy"
image create photo BGPPM -gamma 3.0 -width $screen_w -height $screen_h
BGPPM copy BGPPMsmall -zoom $imgzoomx $imgzoomy

.misc.net create image -10 -10 -image BGPPM -anchor nw

button .next -command { \
	incr groupcount; \
	if {$groupcount > $maxgroup} {incr maxgroup;}; \
	showgroup $groupcount; \
        } -text "Next Group"
pack .next -side left

button .prev -command { \
	set groupcount [expr $groupcount - 1]; \
	if { $groupcount < 1 } { set groupcount 1; }; \
	showgroup $groupcount; \
        } -text "Previous Group"
pack .prev -side left

set groupcount 1
set maxgroup 1

label .gname -text "Current"
pack .gname -side left
label .group -textvariable groupcount -relief sunken -width 5
pack .group -side left

label .gmaxname -text "Last"
pack .gmaxname -side left
label .maxgroup -textvariable maxgroup -relief sunken -width 5
pack .maxgroup -side left

button .psdump -command { \
        .misc.net postscript -file tagedges.ps ; \
        } -text "PS Dump"
pack .psdump -side left

button .save -command { printG; set saved 1} -text "Save"
pack .save -side left -padx 10 -pady 10

# button .quit -command { printG; if { $saved == 1 } { exit; } else { puts "Save first before exiting!"; }; } -text "Save and Quit"
button .quit -command { printG; exit} -text "Save and Quit"
pack .quit -side left -padx 10 -pady 10

button .abort -command { exit } -text "Quit Without Save"
pack .abort -side left -padx 10 -pady 10

#----------------------------------------
# 3. Mouse action binding
#----------------------------------------
bind .misc.net <Button-1> { group %x %y }
bind .misc.net <ButtonPress-1> { set pressed 1 }
bind .misc.net <ButtonRelease-1> { set pressed 0 }
bind .misc.net <Motion> { if { $pressed == 1} { group %x %y;}; if { $pressed2 == 1} { ungroup %x %y;}; }

bind .misc.net <Button-2> { ungroup %x %y }
bind .misc.net <ButtonPress-2> { set pressed2 1 }
bind .misc.net <ButtonRelease-2> { set pressed2 0 }

bind .misc.net <Button-3> {         incr groupcount; \
        if {$groupcount > $maxgroup} {incr maxgroup;}; \
        showgroup $groupcount; bell;}


plotnet
